#!/bin/bash
# Copyright Huawei Technologies Co., Ltd. 2024. All rights reserved.

set -e
SOURCES_DIR="$(cd $(dirname ${0});pwd)"
WORKSHOP="/root"
pushd ${WORKSHOP}
patches_repo=${1}
libs_repo=${2}
url_repos=${3}

function modified_glibc_build() {
    dnf download --source glibc.x86_64
    dnf builddep -y glibc.x86_64
    local _glibc_pkg=$(ls glibc-2.28*)
    rpm -i ${_glibc_pkg} > /dev/null 2>&1
    [ -e "${_glibc_pkg}" ] && rm -rf ${_glibc_pkg}

    pushd rpmbuild/SPECS
    rpmbuild -bp glibc.spec > /dev/null 2>&1
    patch -p0 < ${1}/glibc/glibc.spec.patch
    cp ${1}/glibc/glibc-ld-linux-unwind-exagear.patch ../SOURCES/.
    cp ${1}/glibc/glibc-netlink_request-p64k-exagear.patch ../SOURCES/.
    rpmbuild --nocheck -ba glibc.spec > /dev/null 2>&1
    popd

    _glibc_pkg=$(realpath rpmbuild/BUILD/glibc-2.28*)
    local _build_dir="${_glibc_pkg}/build-x86_64-redhat-linux"
    cp -f ${_build_dir}/elf/ld.so ${_build_dir}/dlfcn/libdl.so ${_build_dir}/libc.so ${2}
    strip ${2}/ld.so
    strip ${2}/libdl.so
    strip ${2}/libc.so
    echo -n EXAGEAR_MODIFIED_v1.0 >> ${2}/ld.so
    echo -n EXAGEAR_MODIFIED_v1.0 >> ${2}/libdl.so
    echo -n EXAGEAR_MODIFIED_v1.0 >> ${2}/libc.so
}

function modified_libgcc_build() {
    dnf download --source libgcc.x86_64
    dnf builddep -y libgcc.x86_64
    dnf install -y glibc.i686 glibc-devel.i686
    local _gcc_pkg=$(ls gcc-8*)
    rpm -i ${_gcc_pkg} > /dev/null 2>&1
    [ -e "${_gcc_pkg}" ] && rm -rf ${_gcc_pkg}

    pushd rpmbuild/SPECS
    rpmbuild -bp gcc.spec > /dev/null 2>&1
    patch -p0 < ${1}/libgcc/gcc.spec.patch
    cp ${1}/libgcc/gcc-exagear.patch ../SOURCES/.
    rpmbuild --nocheck -ba gcc.spec > /dev/null 2>&1
    popd

    _gcc_pkg=$(realpath rpmbuild/BUILD/gcc-8*)
    local _build_dir="${_gcc_pkg}/obj-x86_64-redhat-linux/gcc"
    cp -f ${_build_dir}/libgcc_s.so.1 ${2}
    strip ${2}/libgcc_s.so.1
    echo -n EXAGEAR_MODIFIED_v1.0 >> ${2}/libgcc_s.so.1
}

### yum and epel repo config
mkdir -p /etc/yum.repos.d/bak
mv /etc/yum.repos.d/*.repo /etc/yum.repos.d/bak
if [[ "${url_repos}" =~ "aliyun" ]]; then
    url_repos=${url_repos%/*}
    url_base_repo="${url_repos}/repo/Centos-vault-8.5.2111.repo"
    curl -o /etc/yum.repos.d/CentOS-Base.repo ${url_base_repo}
    sed -i 's/gpgcheck=1/gpgcheck=0/g' /etc/yum.repos.d/CentOS-Base.repo
    sed -i 's/enabled=0/enabled=1/g' /etc/yum.repos.d/CentOS-Base.repo

    cp /etc/yum.repos.d/bak/CentOS-Linux-Sources.repo /etc/yum.repos.d/CentOS-Sources.repo
    offical_src_serv="vault.centos.org"
    url_source_serv=${url_repos#*//}
    sed -i "s/${offical_src_serv}/${url_source_serv}/g" /etc/yum.repos.d/CentOS-Sources.repo
    sed -i 's/$contentdir/centos-vault/g' /etc/yum.repos.d/CentOS-Sources.repo
    sed -i 's/$releasever/8.5.2111/g' /etc/yum.repos.d/CentOS-Sources.repo
    sed -i 's/gpgcheck=1/gpgcheck=0/g' /etc/yum.repos.d/CentOS-Sources.repo
    sed -i 's/enabled=0/enabled=1/g' /etc/yum.repos.d/CentOS-Sources.repo

    url_epel_pkg="${url_repos}/epel/epel-release-latest-8.noarch.rpm"
    trans_proxy="https"
    offical_epel_serv="download.example"
    url_offical_epel="${trans_proxy}://${offical_epel_serv}"
    yum install -y ${url_epel_pkg}
    mv /etc/yum.repos.d/epel-* /etc/yum.repos.d/bak
    sed -i "s|^#baseurl=${url_offical_epel}/pub|baseurl=${url_repos}|" /etc/yum.repos.d/epel*
    sed -i 's|^metalink|#metalink|' /etc/yum.repos.d/epel*
elif [[ "${url_repos}" =~ "huaweicloud" ]]; then
    url_repos=${url_repos%/*}
    url_base_repo="${url_repos}/artifactory/os-conf/centos/centos-8.repo"
    curl -o /etc/yum.repos.d/CentOS-Base.repo ${url_base_repo}
    sed -i 's/https/http/g' /etc/yum.repos.d/CentOS-Base.repo
    sed -i 's/gpgcheck=1/gpgcheck=0/g' /etc/yum.repos.d/CentOS-Base.repo
    sed -i 's/enabled=0/enabled=1/g' /etc/yum.repos.d/CentOS-Base.repo

    cp /etc/yum.repos.d/bak/CentOS-Linux-Sources.repo /etc/yum.repos.d/CentOS-Sources.repo
    offical_src_serv="vault.centos.org"
    url_source_serv=${url_repos#*//}
    sed -i "s/${offical_src_serv}/${url_source_serv}/g" /etc/yum.repos.d/CentOS-Sources.repo
    sed -i 's/$contentdir/centos-vault/g' /etc/yum.repos.d/CentOS-Sources.repo
    sed -i 's/$releasever/8.5.2111/g' /etc/yum.repos.d/CentOS-Sources.repo
    sed -i 's/gpgcheck=1/gpgcheck=0/g' /etc/yum.repos.d/CentOS-Sources.repo
    sed -i 's/enabled=0/enabled=1/g' /etc/yum.repos.d/CentOS-Sources.repo

    yum clean all && yum makecache && yum install -y epel-release
    mv /etc/yum.repos.d/epel-* /etc/yum.repos.d/bak
    sed -i "s/#baseurl/baseurl/g" /etc/yum.repos.d/epel.repo
    sed -i "s/metalink/#metalink/g" /etc/yum.repos.d/epel.repo
    trans_proxy="https"
    offical_epel_serv="download.example"
    url_offical_epel="${trans_proxy}://${offical_epel_serv}"
    sed -i "s|baseurl=${url_offical_epel}/pub|baseurl=${url_repos}|" /etc/yum.repos.d/epel.repo
else
    echo "Un-tested yum-repo url."
    exit 1
fi
yum clean all && yum makecache && yum install -y dnf
dnf install -y rpm-build dnf-plugins-core make

modified_glibc_build ${patches_repo} ${libs_repo}
modified_libgcc_build ${patches_repo} ${libs_repo}

popd
