#!/bin/bash
# Copyright (c) 2019 Huawei Technologies Co., Ltd. All rights reserved.

if [ -z "${EXAGEAR_UBT_BINARY}" ]
then
    echo "Error! Variable EXAGEAR_UBT_BINARY must be set."
    exit 1
fi

EXAGEAR_BIN_PATH=`dirname "${EXAGEAR_UBT_BINARY}"`/ubt

ALL_PROC_EXES=$(find /proc/ -regextype sed -regex '^/proc/[0-9]*/exe$' -type l 2>/dev/null)

EXAGEAR_PIDS=""

for P in ${ALL_PROC_EXES}
do
    EXE_PATH="$(readlink $P)"
    if [ -z "${EXE_PATH}" ]
    then
        continue
	fi
	# Check that executable is inside ExaGear bin folder, but skip optimizer processes
	# (they are either services or can't work without corresponding main UBT process).
	if [[ "${EXE_PATH}" == "${EXAGEAR_BIN_PATH}"* && ! "${EXE_PATH}" == "${EXAGEAR_BIN_PATH}"*"_opt"* ]]
	then
		PID=$(echo $P | awk -F/ '{print $3}')
		EXAGEAR_PIDS+="$PID "
    fi
done

if [ ! -z "${EXAGEAR_PIDS}" ]
then
    if [ ! -z ${EXAGEAR_FORCE_KILL} ]
    then
        # Kill all UBT processes
        for PID in ${EXAGEAR_PIDS}
        do
            kill -9 $PID
        done
    else
        # Print list of UBT processes and exit with error
        echo $'\n'"Error! The following ExaGear processes are running: ${EXAGEAR_PIDS}"
        echo "Please terminate them before uninstalling/upgrading ExaGear."
        echo "Or run uninstalling/upgrading with EXAGEAR_FORCE_KILL=1 environment to forcefully kill all UBT processes."$'\n'
        exit 1
    fi
fi
